/**
 * @(#)MoveResult.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.main;

/**
 * Class represents move result
 * @author Anthony Yakovlev
 */
public class MoveResult {
	/**
	 * Is the game finished
	 */
	public boolean isGameOver = false;
	
	/**
	 * Winner index
	 */
	public int     winner = 0;
	
	/**
	 * Line from which the stones were taken
	 */
	public int     takenFrom = 0;
	
	/**
	 * The amount of stones that were taken
	 */
	public int     nTaken = 0;
	
	/**
	 * Nobody is the winner on current move
	 */
	public static final int WINNER_NOBODY = 0;
	
	/**
	 * Computer is the winner
	 */
	public static final int WINNER_COMPUTER = 1;
	
	/**
	 * Player is the winner
	 */
	public static final int WINNER_PLAYER = 2;
	
	/**
	 * Default constructor of class
	 */
	public MoveResult () {
		isGameOver = false;
		winner = WINNER_NOBODY;
	}
	
	/**
	 * Say that the game is over or not 
	 * @param b - flag
	 */
	public void setGameOver(boolean b) {
		this.isGameOver = b;		
	}
	
	/**
	 * Set id of a winner
	 * @param id - winner
	 */
	public void setWinner (int id) {
		winner = id;
	}
	
	/**
	 * Take stones
	 * @param takeFrom - where to take
	 * @param nTaken - how much to take
	 */
	public void take (int takeFrom, int nTaken) {
		this.takenFrom = takeFrom;
		this.nTaken = nTaken;
	}
	
	/**
	 * Performs array to structure result conversion
	 * @param move the array of amount of stones in lines
	 * @return the parsed structure of null if error
	 */
	public static MoveResult getMoveResult (int [] move) {
		// check correctness
		int posMove = -1;
		for (int i = 0; i < move.length; i++) {
			if (move [i] > 0) {
				if (posMove != -1) {
					return null;					
				}
				posMove = i;
			} else if (move [i] < 0){
				return null;
			}
		}
		
		// parse array in struct
		if (posMove == -1) {
			return null;
		} else {
			MoveResult result = new MoveResult();
			result.setWinner(WINNER_NOBODY);
			result.setGameOver(false);
			result.take(posMove, move [posMove]);
			return result;
		}
	}
}
