/**
 * @(#)ControlPanel.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.screens;

import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import ru.ifmo.knim.main.GameInfo;

/**
 * Represents a panel with plus buttons, minus buttons & edit boxes.
 * 
 * @author Anthony Yakovlev
 */
public class ControlPanel extends JPanel {
	
	/**
	 * Spinners that are used to select moves
	 */
	JSpinner   spinners [];
	
	/**
	 * Game information. Assumed that is passed by pointer.
	 */
	GameInfo   gameInfo;
	
	/**
	 * Constructs a new control panel object.
	 * @param gameInfo the game information to use.
	 * @param width width of control
	 * @param height height of control
	 */
	public ControlPanel (GameInfo gameInfo, int width, int height) {
		spinners = new JSpinner [gameInfo.getLineAmount()];
		
		setLayout(null);
		setSize(width, height);
		
		this.gameInfo = gameInfo;
		height = height / gameInfo.getLineAmount();
		
		for (int i = 0; i < gameInfo.getLineAmount(); i++) {
			spinners [i] = new JSpinner();
			spinners [i].setBounds(0, i * height, width, Math.min(height, 32));
			spinners [i].setModel( new SpinnerNumberModel(0, 0, gameInfo.getFishInLine(i), 1) );
			
			add( spinners [i] );
		}
	
	}
	
	/**
	 * Returns the move, parsing it from the text data
	 * @return fantomas
	 */
	public int [] getMove () {
		int [] mas = new int [spinners.length];
		for (int i = 0; i < spinners.length; i++) {
			mas [i] = ((Integer)spinners [i].getValue()).intValue();			
		}
		
		return mas;
	}

	/**
	 * Updates limits in spinners
	 */
	public void updateLimits() {
		for (int i = 0; i < gameInfo.getLineAmount(); i++) {
			spinners [i].setModel( new SpinnerNumberModel(0, 0, gameInfo.getFishInLine(i), 1) );
		}
	}
}
