/**
 * @(#)FishControlPanel.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.screens;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import ru.ifmo.knim.main.GameInfo;

/**
 * Represents a game control panel.
 * Contains a `stone view` ({@link #controlPanel}) and a control view ({@link #stonePanel}) 
 * Contains resizing policy
 * @author Anthony Yakovlev
 */
public class FishControlPanel extends JPanel {
	/**
	 * Control panel instance.
	 */
	ControlPanel controlPanel;
	
	/**
	 * Stone panel instance.
	 */
	StonePanel   stonePanel;
	
	/**
	 * Image of stone.
	 */
	ImageIcon iconStone;
	
	/**
	 * Default spacing between neighboring cells.
	 */
	int padding;
	
	/**
	 * Fish control panel constructor
	 * @param iconMinus the image for minus sign in control panel
	 * @param iconPlus the image for plus sign in stone panel
	 * @param iconStone the stone image in stone panel 
	 * @param gameInfo the game info instance
	 * @param padding space between neiboring controls 
	 */
	public FishControlPanel(ImageIcon iconStone, GameInfo gameInfo, int padding) {
		// cache common info
		this.iconStone = iconStone;
		this.padding = padding;
		
		// create objects
		setLayout(null);
		redoLayout(gameInfo);
	}
	
	/**
	 * Takes the given amount of stones from stone panel.
	 * @param row the row number, from which the stones will be taken
	 * @param num the amount of stones to take from given row
	 */
	public void takeStones(int row, int num){
		stonePanel.takeStones(row, num);
		controlPanel.updateLimits();
	}
	
	/**
	 * Restarts the stone interface.
	 */
	public void reset() {
		stonePanel.reset();
		controlPanel.updateLimits();
	}
	
	/**
	 * Returns the control panel state.
	 * @return mas
	 */
	public int [] getMove () {
		return controlPanel.getMove();
	}
	
	/**
	 * Perform layout with new game parameters.
	 * @param gameInfo the new game parameters
	 */
	public void redoLayout(GameInfo gameInfo) {
		// clean up
		if (controlPanel != null) {
			remove(controlPanel);
			remove(stonePanel);
		}
		
		// create panel with new properties
		stonePanel   = new StonePanel(gameInfo.getLines(), iconStone, padding);
		controlPanel = new ControlPanel(gameInfo, 45, stonePanel.getHeight());
		
		// setup positions
		stonePanel.setLocation(0, 0);
		controlPanel.setLocation(stonePanel.getWidth() + 1, 0);
		
		// adjust size
		setSize(
				stonePanel.getWidth() + controlPanel.getWidth(), 
				Math.max(stonePanel.getHeight(), controlPanel.getHeight())
		);
		
		// add to control list
		add(controlPanel);
		add(stonePanel);
	}
}
