/**
 * @(#)GameScreen.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.screens;

import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.Parameter;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import ru.ifmo.knim.providers.*;
import ru.ifmo.knim.main.*;

/**
 * Main screen of the application (singleton)
 * 
 * @author Anthony Yakovlev
 * @author Michail Lukin
 */
public class GameScreen extends JFrame implements ActionListener {
	
	static GameScreen             screen = new GameScreen();
	static final int MAX_LAYOUTS = 7;

	ModelEngine                        engine;
	
	JPanel                              contentPane;
	JMenuItem                        exitItem, aboutItem, restartItem, mmoveItem;
	JRadioButtonMenuItem      dffEasyItem, dffHardItem, dffMediumItem;
	JRadioButtonMenuItem [] layType;
	JMenu                              gameMenu, helpMenu, difficultyItem, layoutMenu;
	ButtonGroup             dffGroup, layGroup;
	
	FishControlPanel             gamePanel;
	StatusPanel                   statusPanel;
	
	JMenuBar                        menuBar;
	JButton                         newGameLabel;
	ImageIcon                         iconStone;
	static ResourceBundle                   bundle;
	
	public static GameScreen getScreen() {
		return screen;
	}
	
	private GameScreen() {
		
		URL url = getClass().getResource("/stone.gif" ); // There was a historical c:\test\stone.gif...
		
		// old code
		if (url == null) {
			iconStone = new ImageIcon("c:\\temp\\stone.gif");
		}
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		jbInit();
	}
	
	/**
	 * Makes apropriate changes according to a move
	 * @param move
	 */
	public void takeStones(MoveResult move) {
		GamePlay.getGamePlay().getGameInfo().takeStones(move.takenFrom, move.nTaken);
		gamePanel.takeStones(move.takenFrom, move.nTaken);
	}
	
	public void actionPerformed (ActionEvent e) {
		//
		// Catch menu items
		//
		if (e.getSource() == exitItem) {
			fireEvent(InterfaceEventProvider.E6, null);
		} else if (e.getSource() == aboutItem) {
			URL url = getClass().getResource("stone.gif" ); // There was a historical c:\test\stone.gif...
			fireEvent(InterfaceEventProvider.E5, null);                  
		} else if (e.getSource() == restartItem) { 
			fireEvent(InterfaceEventProvider.E2, null);
		}
		
		//
		// Perform a move
		//
		if (e.getSource() == mmoveItem || e.getSource() == newGameLabel) {
			fireEvent(InterfaceEventProvider.E3, new Parameter("Move", getMove()));
		}
		
		//
		// Select difficulty
		//
		
		GameInfo gameInfo = GamePlay.getGamePlay().getGameInfo();
		if (e.getSource() == dffEasyItem) {
			gameInfo.setGetDifficulty(GameInfo.DIFFICULTY_EASY);
			fireEvent(InterfaceEventProvider.E4, new Parameter("Difficulty", new Integer(GameInfo.DIFFICULTY_EASY)));
		} else if (e.getSource() == dffHardItem){
			gameInfo.setGetDifficulty(GameInfo.DIFFICULTY_HARD);
			fireEvent(InterfaceEventProvider.E4, new Parameter("Difficulty", new Integer(GameInfo.DIFFICULTY_HARD)));
		} else if (e.getSource() == dffMediumItem) {
			gameInfo.setGetDifficulty(GameInfo.DIFFICULTY_MEDIUM);
			fireEvent(InterfaceEventProvider.E4, new Parameter("Difficulty", new Integer(GameInfo.DIFFICULTY_MEDIUM)));
		}
		
		int [] [] config = {
				{3, 4, 5},
				{1, 2},
				{2, 3, 9},
				{1, 3, 5, 7},
				{4, 8, 12, 12},
				{2, 4, 6, 8, 10},
				{4, 6, 6, 7, 8, 9, 10, 11, 12}
		};

		
		for (int i = 0; i < MAX_LAYOUTS; i++) {
			if (e.getSource() == layType [i]) {
				gameInfo.setStones(config [i]);
				gamePanel.redoLayout(gameInfo);
				performLayout();
				fireEvent(InterfaceEventProvider.E7, null);
			}
		}
	}
	
	/**
	 * Fires event to apropriate handler
	 * @param eventName - the event name
	 * @param parameter - the parameters
	 */
	private void fireEvent(String eventName, Parameter parameter) {
		if (engine != null) {
			if (parameter == null) {
				engine.getEventManager().handle(new Event(eventName), StateMachineContextImpl.create());
			} else {
				engine.getEventManager().handle(new Event(eventName, new Parameter[]{parameter}), StateMachineContextImpl.create());
			}
		}
	}
	
	/**
	 * Inits connector with state machine and event engine
	 */
	public void init(ModelEngine engine) {
		this.engine = engine;
		fireEvent(InterfaceEventProvider.E2, null);
	}
	
	/**
	 * Gavna kusok 
	 */
	private void jbInit() {
		//
		// Get content pane
		//
		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(null);
		
		gamePanel = new FishControlPanel(iconStone, GamePlay.getGamePlay().getGameInfo(), 10);
		gamePanel.setLocation(10, 10);
		contentPane.add(gamePanel);
		
		//
		// Customize frame
		//
		setResizable(false);
		setTitle("  "); 
		
		//
		// Customize menus
		//
		menuBar = new JMenuBar();
		gameMenu = new JMenu("");
		helpMenu = new JMenu("C");
		gameMenu.setMnemonic('g');
		helpMenu.setMnemonic('H');

		//
		// Customize menu items
		//
		exitItem = new JMenuItem("");
		exitItem.setMnemonic('e');
		exitItem.setAccelerator(KeyStroke.getKeyStroke('E'));
		
		aboutItem = new JMenuItem(" ");
		aboutItem.setMnemonic('a');
		aboutItem.setAccelerator(KeyStroke.getKeyStroke('A'));
		
		restartItem = new JMenuItem(" ");
		restartItem.setMnemonic('r');
		restartItem.setAccelerator(KeyStroke.getKeyStroke('R'));
		
		mmoveItem = new JMenuItem(" ");
		mmoveItem.setMnemonic('g');
		mmoveItem.setAccelerator(KeyStroke.getKeyStroke('G'));
		
		difficultyItem = new JMenu("");
		dffEasyItem = new JRadioButtonMenuItem(" ( 2 )");
		dffEasyItem.setSelected(true);
		dffHardItem = new JRadioButtonMenuItem(" ( )");
		dffMediumItem = new JRadioButtonMenuItem(" ( 1 )");
		
		String [] config = {
			" (3-4-5)",
			" (1-2)",
			" (2-3-9)",
			" (1-3-5-7)",
			"- (4-8-12-12)",
			" (2-4-6-8-10)",
			" (4-6-6-7-8-9-9-10-11-12)",
		};

		layGroup = new ButtonGroup();
		dffGroup = new ButtonGroup();
		
		layType = new JRadioButtonMenuItem [MAX_LAYOUTS];
		for (int i = 0; i < MAX_LAYOUTS; i++) {            
			layType [i] = new JRadioButtonMenuItem(config [i]);
			layGroup.add( layType[i] );
		}
		layType[0].setSelected(true);
		
		dffGroup.add(dffEasyItem);
		dffGroup.add(dffHardItem);
		dffGroup.add(dffMediumItem);
		
		layoutMenu = new JMenu("");
		
		exitItem.addActionListener(this);
		aboutItem.addActionListener(this);
		restartItem.addActionListener(this);
		mmoveItem.addActionListener(this);
		dffEasyItem.addActionListener(this);
		dffHardItem.addActionListener(this);
		dffMediumItem.addActionListener(this);
		layoutMenu.addActionListener(this);
		
		for (int i = 0; i < MAX_LAYOUTS; i++) {
			layType [i].addActionListener(this);
			layoutMenu.add( layType[i] );
		}
		
		difficultyItem.add(dffEasyItem);
		difficultyItem.add(dffMediumItem);
		difficultyItem.add(dffHardItem);
		
		gameMenu.add(restartItem);
		gameMenu.add(new JSeparator());
		menuBar.add(gameMenu);
		menuBar.add(helpMenu);
		gameMenu.add(mmoveItem);
		gameMenu.add(difficultyItem);
		gameMenu.add(layoutMenu);
		gameMenu.add(new JSeparator());
		gameMenu.add(exitItem);
		helpMenu.add(aboutItem);
		
		setJMenuBar(menuBar);
		
		//
		// Add images
		// 
		newGameLabel = new JButton(" ");
		contentPane.add(newGameLabel);
		newGameLabel.addActionListener(this);

		statusPanel = new StatusPanel(gamePanel.getWidth() + gamePanel.getX() + 20, 18);
		statusPanel.updateStatus(" .");
		contentPane.add(statusPanel);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		restartIface();
	}
	
	/**
	 * Gets move.
	 * @return
	 */
	public int [] getMove () {
		return gamePanel.getMove();
	}
	
	/**
	 * Hello.
	 */
	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			fireEvent(InterfaceEventProvider.E6, null);
		}
	}

	/**
	 * Restarts all interface to initial states 
	 */
	public void restartIface () {
		GamePlay.getGamePlay().getGameInfo().restartGame();
		gamePanel.reset();
		performLayout();
		
		// push timer 
		fireEvent(GameEventProvider.E24, null);
	}
	
	/**
	 * Updates status bar text
	 * @param text - a new text for the status bar
	 */
	public void updateStatus (String text) {
		statusPanel.updateStatus(text);
	}
	
	public void updateStatus(MoveResult move) {
		updateStatus("  ,  " + move.nTaken + "   " + (move.takenFrom + 1) + " ");                        
	}
	
	public void performLayout() {
		newGameLabel.setBounds(gamePanel.getX(), gamePanel.getHeight() + gamePanel.getY(), gamePanel.getWidth() - gamePanel.getX() + 10, 50);

		statusPanel.updateStatus(" .");
		statusPanel.setLocation(0, newGameLabel.getY() + newGameLabel.getHeight() + 2);
		statusPanel.setSize(gamePanel.getWidth() + gamePanel.getX() + 20, 18);
		
		setSize(gamePanel.getWidth() + gamePanel.getX() + 20, 
				gamePanel.getY() + gamePanel.getHeight() + statusPanel.getHeight() + newGameLabel.getHeight() + 50);
		
		setVisible(true);
	}
}

