/**
 * @(#)StatusPanel.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.screens;

import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;

/**
 * Status panel with capability to draw separating line & to display text message.
 * @author Anthony Yakovlev
 */
public class StatusPanel extends JPanel {
	/**
	 * Storage for last status. 
	 */
	JLabel		statusLabel;
	
	/**
	 * Line, separating this control from uppper content.
	 */
	JSeparator	separator;
	
	/**
	 * Constructor. Lays out everything and sets sizes. 
	 * @param width the desired width of control
	 * @param height the desired height of control
	 */
	public StatusPanel (int width, int height) {
		setLayout(null);
		statusLabel = new JLabel();
		statusLabel.setLocation(0, 5);
		statusLabel.setFont(new Font("courier new", Font.PLAIN, 11));
		
		separator = new JSeparator();
		separator.setLocation(0, 0);
		
		setSize(width, height);
		
		add(statusLabel);
		add(separator);
	}
	
	/**
	 * Updates status of control.
	 * @param text the message to display
	 */
	public void updateStatus(String text) {
		statusLabel.setText(text);		
	}
	
	/**
	 * Resizes status panel.
	 */
	public void setSize(int width, int height) {
		statusLabel.setSize(width, height - 5);
		separator.setSize(width, 5);
		super.setSize(width, height);
	}
}
