/**
 * @(#)StonePanel.java
 * 
 *  Copyright Anthony Yakovlev <yakovlev@rain.ifmo.ru> and Michail Lukin <michail@users.msn.com>
 */

package ru.ifmo.knim.screens;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Represents a panel with stones 
 * @author Anthony Yakovlev
 */
public class StonePanel extends JPanel {
	
	/**
	 * Array of labels for stones.
	 */
	JLabel [] [] stones;

	/**
	 * Constructs a new stone panel object.
	 * @param dimensions the array with stones amount per each line
	 * @param fishImage image to use as a picture of stone
	 * @param padding the spacing between stones
	 */
	public StonePanel (int [] dimensions, ImageIcon fishImage, int padding) {
		stones = new JLabel [dimensions.length] [];
		int maxStones = -239;
		setLayout(null);
		
		int width = fishImage.getIconWidth();
		int height = fishImage.getIconHeight();
	
		for (int i = 0; i < dimensions.length; i++) {
			
			if (dimensions [i] > maxStones) {
				maxStones = dimensions [i];
			}
			
			stones [i] = new JLabel [dimensions [i] ];
			for (int j = 0; j < dimensions [i]; j++) {
				stones [i] [j] = new JLabel(fishImage);
				stones [i] [j].setBounds(j * (width + padding), i * (height + padding), width, height);
				add(stones [i] [j]);
			}
		}
		
		setSize(maxStones * (width + padding), (height + padding) * dimensions.length);
	}
	
	/**
	 * Takes stones from given row.
	 * @param row the row to take stones
	 * @param num the amount of stones to take
	 */
	public void takeStones(int row, int num){
		int pos = stones [row].length - 1;
		while (pos > -1 && num > 0) {
			if (stones [row] [pos].isVisible()) {
				stones [row] [pos].setVisible(false);
				num--;
			}
			pos--;
		}
	}
	
	/**
	 * Shows all hidden stones. 
	 */
	public void reset() {
		for (int i = 0; i < stones.length; i++) {
			for (int j = 0; j < stones [i].length; j++) {
				stones [i] [j].setVisible(true);
			}
		}
	}
}
